part of model_notifier;

abstract class ApiCollectionModel<T> extends ValueModel<List<T>>
    with ListModelMixin<T>
    implements StoredModel<List<T>> {
  ApiCollectionModel(this.endpoint, [List<T>? value]) : super(value ?? []);

  @override
  bool get notifyOnChangeList => false;

  @override
  bool get notifyOnChangeValue => true;

  final String endpoint;

  String get getEndpoint => endpoint;

  String get postEndpoint => endpoint;

  Map<String, String> get getHeaders => {};

  Map<String, String> get postHeaders => {};

  List<T> fromCollection(List<Object> list);
  List<Object> toCollection(List<T> list);

  @protected
  @mustCallSuper
  Future<void> onLoad() async {}

  @protected
  @mustCallSuper
  Future<void> onSave() async {}

  @protected
  @mustCallSuper
  Future<void> onDidLoad() async {}

  @protected
  @mustCallSuper
  Future<void> onDidSave() async {}

  @protected
  @mustCallSuper
  void onCatchResponse(Response response) {}

  @protected
  List<Object> fromResponse(String json) => jsonDecodeAsList(json);

  @protected
  dynamic toRequest(List<Object> list) => jsonEncode(list);

  @protected
  @mustCallSuper
  List<Object> filterOnLoad(List<Object> loaded) => loaded;

  @protected
  @mustCallSuper
  List<Object> filterOnSave(List<Object> save) => save;

  @protected
  T createDocument();

  T create() => createDocument();

  @override
  Future<List<T>> load() async {
    await onLoad();
    final res = await get(Uri.parse(getEndpoint), headers: getHeaders);
    onCatchResponse(res);
    final data = fromCollection(filterOnLoad(fromResponse(res.body)));
    addAll(data);
    streamController.sink.add(value);
    notifyListeners();
    await onDidLoad();
    return this;
  }

  @override
  Future<List<T>> save() async {
    await onSave();
    final res = await post(
      Uri.parse(postEndpoint),
      headers: postHeaders,
      body: toRequest(filterOnSave(toCollection(this))),
    );
    onCatchResponse(res);
    streamController.sink.add(value);
    notifyListeners();
    await onDidSave();
    return this;
  }
}
